@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2018 nickolas360 <contact@nickolas360.com>
+# Copyright (C) 2017-2019 nickolas360 <contact@nickolas360.com>
 #
 # This file is part of Harmony.
 #
@@ -25,6 +25,7 @@ import re
 import sys
 
 BASE_URL = "https://discordapp.com/api/v6/"
+CLIENT_BUILD_NUMBER = 35116
 
 
 class GenericResponse:
@@ -41,6 +42,15 @@ class GenericResponse:
     def needs_captcha(self):
         return not self.success and "captcha_key" in self.json
 
+    @property
+    def invalid_captcha(self):
+        if self.success:
+            return False
+        try:
+            return "incorrect-captcha-sol" in self.json["captcha_key"]
+        except (AttributeError, IndexError, KeyError, TypeError):
+            return False
+
     @property
     def ratelimited(self):
         return self.status_code == 429
@@ -306,10 +316,15 @@ class Discord:
         self.super_properties = super_properties
         self._debug = debug
 
-        self.super_properties.setdefault("referrer", "")
-        self.super_properties.setdefault("referring_domain", "")
-        self.super_properties.setdefault("referrer_current", "")
-        self.super_properties.setdefault("referring_domain_current", "")
+        sprops = self.super_properties
+        sprops.setdefault("browser_user_agent", user_agent)
+        sprops.setdefault("referrer", "")
+        sprops.setdefault("referring_domain", "")
+        sprops.setdefault("referrer_current", "")
+        sprops.setdefault("referring_domain_current", "")
+        sprops.setdefault("release_channel", "stable")
+        sprops.setdefault("client_build_number", CLIENT_BUILD_NUMBER)
+        sprops.setdefault("client_event_source", None)
 
         self._token = None
         self.fingerprint = None
@@ -454,6 +469,7 @@ class Discord:
             "password": password,
             "invite": invite,
             "captcha_key": captcha_key,
+            "consent": True,
         }, referer="register?redirect=%2F")
 
         resp = ResponseWithToken(r)
@@ -562,6 +578,11 @@ class Discord:
         resp = ServerDetailsResponse(r)
         return resp
 
+    def leave_server(self, server_id):
+        r = self.delete("users/@me/guilds/{}".format(server_id), auth=True)
+        resp = GenericResponse(r)
+        return resp
+
     def server_members(self, server_id, limit=1000):
         r = self.get(
             "guilds/{}/members?limit={}".format(server_id, limit), auth=True)
