@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2018 nickolas360 <contact@nickolas360.com>
+# Copyright (C) 2017-2019 nickolas360 <contact@nickolas360.com>
 #
 # This file is part of Harmony.
 #
@@ -16,14 +16,16 @@
 # along with Harmony.  If not, see <http://www.gnu.org/licenses/>.
 
 from . import discord
+from . import messages
 from .discord import Discord, FriendPolicy
 from .user_agents import random_user_agent
-from librecaptcha import get_token
 
+import librecaptcha
 from PIL import Image
 
 from io import BytesIO
 import base64
+import builtins
 import functools
 import getpass as getpass_module
 import json
@@ -33,35 +35,40 @@ import re
 import sys
 import traceback
 
-__version__ = "0.5.0"
+__version__ = "0.6.0"
 RECAPTCHA_API_KEY = "6Lef5iQTAAAAAKeIvIY-DeexoO3gj7ryl9rLMEnn"
 RECAPTCHA_SITE_URL = "https://discordapp.com"
 
 INTERACTIVE_HELP = """\
 Commands:
-  help          Print this help message.
-  quit          Quit the program.
-  log-in        Log in.
-  log-out       Log out.
-  register      Register a new account.
-  verify        Verify your email address.
+          help  Print this help message.
+          quit  Quit the program.
+        log-in  Log in.
+       log-out  Log out.
+      register  Register a new account.
+        verify  Verify your email address.
   authorize-ip  Authorize a new login location.
-  resend        Resend the verification email.
-  tag           Get your account tag.
-  get-details   Get your current account details.
-  set-details   Update account username, email, password, or avatar.
+        resend  Resend the verification email.
+           tag  Get your account tag.
+   get-details  Get your current account details.
+   set-details  Update account username, email, password, or avatar.
   get-settings  Get account settings.
   set-settings  Update account settings. (Not all settings are supported.)
-  delete        Delete your account.
-  undelete      Undelete an account marked for deletion.
-  show-invite   Show and optionally accept a server invite.
-  servers       List the servers you're in.
-  members       List the members of a server.
-  transfer      Transfer a server to another user.
-  rm-server     Delete a server.
+        delete  Delete your account.
+      undelete  Undelete an account marked for deletion.
+   show-invite  Show and optionally accept a server invite.
+       servers  List the servers you're in.
+  leave-server  Leave a server.
+       members  List the members of a server.
+      transfer  Transfer a server to another user.
+     rm-server  Delete a server.
 """
 
 
+def stderr(*args, **kwargs):
+    print(*args, **kwargs, file=sys.stderr)
+
+
 def getpass(prompt="Password: ", stream=None):
     if sys.stdin.isatty():
         return getpass_module.getpass(prompt, stream)
@@ -95,46 +102,46 @@ def pluralize(number, singular, plural=None):
 
 
 def print_errors(response, message=None, file=sys.stdout):
-    def _print(*args, **kwargs):
-        print(*args, file=file, **kwargs)
+    def print(*args, **kwargs):
+        builtins.print(*args, file=file, **kwargs)
 
     if response is None or response.success:
-        _print(message, "(no errors)")
+        print(message, "(no errors)")
         return
 
     if response.ratelimited:
         retry_sec = math.ceil(response.retry_ms / 1000)
-        _print(message)
-        _print("  Request to server was blocked due to rate limiting.")
-        _print("  Try again in {} seconds.".format(retry_sec))
+        print(message)
+        print("  Request to server was blocked due to rate limiting.")
+        print("  Try again in {} seconds.".format(retry_sec))
         return
 
     if response.json is None:
-        _print(message)
-        _print("  Unknown error (status code {})".format(response.status_code))
+        print(message)
+        print("  Unknown error (status code {})".format(response.status_code))
         return
 
     data = response.json
     if isinstance(data, list):
-        _print(message)
+        print(message)
         for message in data:
-            _print(" " * 2 + str(message))
+            print(" " * 2 + str(message))
         return
 
     if not isinstance(data, dict):
-        _print(message)
-        _print(data)
+        print(message)
+        print(data)
         return
 
     if not all(isinstance(value, list) for value in data.values()):
-        _print(message, json.dumps(data, indent=2))
+        print(message, json.dumps(data, indent=2))
         return
 
     for i, (name, errors) in enumerate(data.items()):
-        _print(message)
-        _print(" " * 2 + "{}:".format(name))
+        print(message)
+        print(" " * 2 + "{}:".format(name))
         for error in errors:
-            _print(" " * 4 + str(error))
+            print(" " * 4 + str(error))
 
 
 class CommandFailure(Exception):
@@ -175,20 +182,23 @@ def explicit_filter_to_str(explicit_filter):
     }.get(explicit_filter, explicit_filter.name)
 
 
-class InteractiveDiscord:
+class DiscordCli:
     def __init__(self, discord=None, debug=False):
+        if discord is None:
+            browser_ua, super_properties = random_user_agent()
+            discord = Discord(browser_ua, super_properties, debug=debug)
         self.dc = discord
         self.debug = debug
-        if not self.dc:
-            browser_ua, super_properties = random_user_agent()
-            self.dc = Discord(browser_ua, super_properties, debug=debug)
+
+    @property
+    def user_agent(self):
+        return self.dc.user_agent
 
     def get_command_function(self, command):
         try:
             command = command.split(None, 1)[0]
         except IndexError:
             return None
-
         return {
             "help": self.print_help,
             "quit": sys.exit,
@@ -207,6 +217,7 @@ class InteractiveDiscord:
             "undelete": self.undelete,
             "show-invite": self.show_invite,
             "servers": self.servers,
+            "leave-server": self.leave_server,
             "members": self.server_members,
             "transfer": self.transfer_server,
             "rm-server": self.delete_server,
@@ -218,24 +229,24 @@ class InteractiveDiscord:
         print(INTERACTIVE_HELP, end="", file=file)
 
     def command_loop(self):
-        print('Type "help" for a list of commands.', file=sys.stderr)
+        stderr('Type "help" for a list of commands.')
         while True:
             try:
                 self.exec_single_command()
             except EOFError:
-                print(file=sys.stderr)
+                stderr()
                 sys.exit()
             except KeyboardInterrupt:
-                print(file=sys.stderr)
+                stderr()
 
     def exec_single_command(self):
-        print("> ", end="", file=sys.stderr, flush=True)
+        stderr("> ", end="", flush=True)
         command = input()
         if not command:
             return False
         func = self.get_command_function(command)
         if func is None:
-            print('Unknown command. Type "help" for help.\n', file=sys.stderr)
+            stderr('Unknown command. Type "help" for help.\n')
             return False
 
         success = True
@@ -245,98 +256,93 @@ class InteractiveDiscord:
             success = False
         except EOFError:
             raise
-        except Exception as e:
-            print("\nError encountered while running the command:\n",
-                  file=sys.stderr)
+        except Exception:
+            stderr("\nError encountered while running the command:\n")
             traceback.print_exc()
-            print(file=sys.stderr)
+            stderr()
             return False
-
-        print(file=sys.stderr)
+        stderr()
         return success
 
-    def get_captcha_key(self):
-        print("You must solve a CAPTCHA challenge.")
-        print("You will be guided through the steps necessary to complete the "
-              "challenge.")
-        print("Press enter to start the challenge...")
-        input()
+    def try_request(self, func):
+        try:
+            return func()
+        except discord.RequestError as e:
+            return e.response
 
+    def try_or_error(self, error_message, func, *args, throw=True):
+        response = self.try_request(func)
+        self.handle_response(error_message, response, throw=throw)
+        return response
+
+    def handle_response(self, error_message, response, *, throw):
+        if response.success:
+            return
+        if error_message is not None:
+            print()
+            print_errors(response, "{} Errors:".format(error_message))
+        if throw:
+            raise CommandFailure
+
+    def get_captcha_key(self, *, ask: bool):
+        if ask:
+            print(messages.MUST_SOLVE_CAPTCHA, end="")
+            input()
         while True:
             try:
-                token = get_token(
-                    RECAPTCHA_API_KEY, RECAPTCHA_SITE_URL, debug=self.debug,
+                token = librecaptcha.get_token(
+                    RECAPTCHA_API_KEY, RECAPTCHA_SITE_URL, self.user_agent,
+                    gui=librecaptcha.has_gui(), debug=self.debug,
                 )
             except Exception:
-                print("\nError encountered while solving the CAPTCHA:\n",
-                      file=sys.stderr)
+                stderr("\nError encountered while solving the CAPTCHA:\n")
                 traceback.print_exc()
-                print(file=sys.stderr)
-                if not ask_yn("Try another captcha?"):
+                stderr()
+                if not ask_yn("Try another CAPTCHA?"):
                     print("CAPTCHA challenge failed.")
                     return None
                 continue
             print("Successfully solved the CAPTCHA challenge.")
             return token
 
-    def try_with_captcha(
-            self, error_message, func, *args, throw=True, forward_exc=False,
-            **kwargs):
-        response, exception = self.try_request(func, *args, **kwargs)
+    def try_once_with_captcha(
+            self, error_message, func, *, throw=True, ask=True):
+        response = self.try_request(lambda: func(None))
         if not response.needs_captcha:
-            self.handle_response(
-                error_message, response, exception, throw, forward_exc)
+            self.handle_response(error_message, response, throw=throw)
             return response
 
-        captcha_key = self.get_captcha_key()
+        captcha_key = self.get_captcha_key(ask=ask)
         if captcha_key is None:
             if error_message is not None:
                 print()
                 print(error_message)
             return response
-
         return self.try_or_error(
-            error_message, func, *args, throw=throw, forward_exc=forward_exc,
-            captcha_key=captcha_key, **kwargs,
+            error_message, lambda: func(captcha_key), throw=throw,
         )
 
-    def try_or_error(
-            self, error_message, func, *args, throw=True, forward_exc=False,
-            **kwargs):
-        response, exception = self.try_request(func, *args, **kwargs)
-        self.handle_response(
-            error_message, response, exception, throw, forward_exc)
-        return response
-
-    def try_request(self, func, *args, **kwargs):
-        try:
-            return func(*args, **kwargs), None
-        except discord.RequestError as e:
-            return e.response, e
-
-    def handle_response(
-            self, error_message, response, exception, throw, forward_exc):
-        if response.success:
-            return
-        if error_message is not None:
-            print()
-            print_errors(response, "{} Errors:".format(error_message))
-        if exception is not None:
-            if forward_exc:
-                raise exception
-            raise CommandFailure
-        if throw:
-            raise CommandFailure
+    def try_with_captcha(self, *args, **kwargs):
+        ask = True
+        while True:
+            response = self.try_once_with_captcha(*args, ask=ask, **kwargs)
+            ask = False
+            if not response.invalid_captcha:
+                return response
+            print("\n" + messages.INVALID_CAPTCHA)
+            if not ask_yn("Try another CAPTCHA?", default_yes=True):
+                return response
 
     @warn_if_logged_in
     def log_in(self, undelete=False):
         email = input_nb("Email address: ")
         password = getpass()
+        print()
         response = self.try_with_captcha(
-            "Login failed.", self.dc.log_in, email, password,
-            undelete=undelete, throw=False,
+            "Login failed.", lambda key: self.dc.log_in(
+                email, password, undelete=undelete, captcha_key=key
+            ), throw=False,
         )
-
         if response.success:
             print("You are now logged in.")
             return
@@ -368,28 +374,29 @@ class InteractiveDiscord:
                 break
             print("Passwords do not match.")
 
+        print()
         self.try_with_captcha(
-            "Registration failed.", self.dc.register, email=email,
-            username=username, password=password)
-        print("You have successfully registered.")
-        print('You should verify your email address with the "verify" '
-              "command.")
+            "Registration failed.", lambda key: self.dc.register(
+                email=email, username=username, password=password,
+                captcha_key=key,
+            ),
+        )
+        print(messages.SUCCESSFUL_REGISTRATION, end="")
 
     @warn_if_logged_in
     def verify_email(self):
-        print("Enter the verification link you received by email.")
-        print("The email should contain a link that looks like this:")
-        print("https://discordapp.com/verify?token=<token>")
+        print(messages.ENTER_VERIFICATION_LINK, end="")
         link = input_nb("Enter the link: ")
-
         match = re.search(r"token=([A-Za-z0-9_\.\-\+]+)", link)
         if match is None:
             print("Could not extract token from link.")
             raise CommandFailure
         token = match.group(1)
-
+        print()
         self.try_with_captcha(
-            "Verification failed.", self.dc.verify_email, token)
+            "Verification failed.",
+            lambda key: self.dc.verify_email(token, captcha_key=key),
+        )
         print("Your email address is now verified.")
         print("You are now logged in.")
 
@@ -397,32 +404,32 @@ class InteractiveDiscord:
     def resend_verification_email(self):
         self.try_or_error(
             "Could not send verification email.",
-            self.dc.resend_verification_email)
+            self.dc.resend_verification_email,
+        )
         print("Resent verification email.")
 
     @warn_if_logged_in
     def authorize_ip(self):
-        print("Enter the new location verification link you received by "
-              "email.")
-        print("The email should contain a link that looks like this:")
-        print("https://discordapp.com/authorize-ip?token=<token>")
+        print(messages.ENTER_NEW_LOCATION_LINK, end="")
         link = input_nb("Enter the link: ")
-
         match = re.search(r"token=([A-Za-z0-9_\.\-\+]+)", link)
         if match is None:
             print("Could not extract token from link.")
             raise CommandFailure
         token = match.group(1)
-
+        print()
         self.try_with_captcha(
-            "Verification failed.", self.dc.authorize_ip, token)
+            "Verification failed.",
+            lambda key: self.dc.authorize_ip(token, captcha_key=key),
+        )
         print("New location verified. You may now log in.")
 
     @needs_auth
     def get_tag(self):
         response = self.try_or_error(
             "Could not get account tag.", self.dc.get_account_details,
-            cached=True)
+            cached=True,
+        )
         print(response.tag)
 
     @needs_auth
@@ -439,7 +446,8 @@ class InteractiveDiscord:
     @needs_auth
     def set_account_details(self):
         response = self.try_or_error(
-            "Could not get account details.", self.dc.get_account_details)
+            "Could not get account details.", self.dc.get_account_details,
+        )
         params = response.to_params()
         if ask_yn("Change password?"):
             while True:
@@ -474,7 +482,8 @@ class InteractiveDiscord:
         password = getpass("Current password: ")
         response = self.try_or_error(
             "Account update failed.", self.dc.set_account_details,
-            params=params, password=password)
+            params=params, password=password,
+        )
 
         print("Account details updated.")
         if not response.verified_email:
@@ -497,7 +506,8 @@ class InteractiveDiscord:
               to_yn(FriendPolicy.has_mutual_friends(policy)))
         print("  Server members:",
               to_yn(FriendPolicy.has_server_members(policy)))
-        print("  Everyone:", to_yn(FriendPolicy.has_all(policy)))
+        print("  Everyone:",
+              to_yn(FriendPolicy.has_all(policy)))
 
     @needs_auth
     def set_settings(self):
@@ -529,7 +539,8 @@ class InteractiveDiscord:
                 params.friend_policy |= FriendPolicy.ALL
 
         self.try_or_error(
-            "Settings update failed.", self.dc.set_settings, params)
+            "Settings update failed.", self.dc.set_settings, params,
+        )
         print("Settings updated.")
 
     @needs_auth
@@ -539,7 +550,8 @@ class InteractiveDiscord:
         password = getpass("Account password: ")
         response = self.try_or_error(
             "Account deletion failed.", self.dc.delete_account, password,
-            throw=False)
+            throw=False,
+        )
 
         if response.success:
             print("Account scheduled for deletion.")
@@ -560,31 +572,32 @@ class InteractiveDiscord:
         print("Enter the invite link.")
         print("It should look like this: https://discord.gg/<id>")
         link = input_nb("Enter the link: ")
-
         match = re.search(r"/([A-Za-z0-9_\.\-\+]+)/*$", link)
         if match is None:
             print("Could not extract invite ID from link.")
             return False
         invite_id = match.group(1)
         invite = self.try_or_error(
-            "Could not get invite details.", self.dc.invite_details, invite_id)
+            "Could not get invite details.", self.dc.invite_details, invite_id,
+        )
 
         print('\n{} has invited you to join the server "{}".'.format(
-            invite.inviter_tag, invite.server_name))
+            invite.inviter_tag, invite.server_name,
+        ))
         if invite.member_count is not None:
             print("The server has approximately {} {}.".format(
-                invite.member_count, pluralize(invite.member_count, "member")))
+                invite.member_count, pluralize(invite.member_count, "member"),
+            ))
         if not ask_yn("Accept invite?"):
             return
-
         self.try_or_error(
-            "Could not accept invite.", self.dc.accept_invite, invite_id)
+            "Could not accept invite.", self.dc.accept_invite, invite_id,
+        )
         print("Invite accepted.")
 
     @needs_auth
     def servers(self):
-        response = self.try_or_error(
-            "Could not get servers.", self.dc.servers)
+        response = self.try_or_error("Could not get servers.", self.dc.servers)
         if not response.servers:
             print("You are not in any servers.")
             return
@@ -596,19 +609,21 @@ class InteractiveDiscord:
 
     def print_servers(self, owned=False, prefix=""):
         servers_resp = self.try_or_error(
-            prefix + "Could not get servers.", self.dc.servers)
+            prefix + "Could not get servers.", self.dc.servers,
+        )
         print(prefix, end="")
 
         servers = servers_resp.servers
         if owned:
             servers = [s for s in servers_resp.servers if s.is_owner]
         if not servers:
-            print("You don't own any servers." if owned else
-                  "You're not in any servers.")
+            print(
+                "You don't own any servers." if owned else
+                "You're not in any servers.",
+            )
             return False
 
-        print("Your servers:" if owned else
-              "Servers you're in:")
+        print("Your servers:" if owned else "Servers you're in:")
         for server in servers:
             print("  {} [ID: {}]".format(server.name, server.id))
         return True
@@ -616,11 +631,28 @@ class InteractiveDiscord:
     def print_members(self, server_id, prefix=""):
         members_resp = self.try_or_error(
             prefix + "Could not get server members.", self.dc.server_members,
-            server_id)
+            server_id,
+        )
         print(prefix + "Members in this server:")
         for member in members_resp.members:
             print("  {} [ID: {}]".format(member.tag, member.id))
 
+    @needs_auth
+    def leave_server(self):
+        if not self.print_servers(owned=False):
+            return
+        server_id = input_nb("\nEnter the ID of the server to leave: ")
+        details = self.try_or_error(
+            "Could not get server details.", self.dc.server_details, server_id,
+        )
+        print('You are about to leave the server "{}".'.format(details.name))
+        if not ask_yn("Are you sure you want to leave this server?"):
+            return
+        self.try_or_error(
+            "Could not leave server.", self.dc.leave_server, server_id,
+        )
+        print("Left server.")
+
     @needs_auth
     def server_members(self):
         if not self.print_servers(owned=False):
@@ -634,11 +666,11 @@ class InteractiveDiscord:
             return
         server_id = input_nb("\nEnter the ID of the server to transfer: ")
         self.print_members(server_id, prefix="\n")
-
         owner_id = input_nb("\nEnter the ID of the new server owner: ")
         self.try_or_error(
             "Could not transfer server.", self.dc.transfer_server,
-            server_id, owner_id)
+            server_id, owner_id,
+        )
         print("Server transferred.")
 
     @needs_auth
@@ -646,15 +678,15 @@ class InteractiveDiscord:
         if not self.print_servers(owned=True):
             return
         server_id = input_nb("\nEnter the ID of the server to delete: ")
-
         details = self.try_or_error(
-            "Could not get server details.", self.dc.server_details, server_id)
+            "Could not get server details.", self.dc.server_details, server_id,
+        )
         print('You are about to delete the server "{}".'.format(details.name))
         if not ask_yn("Are you sure you want to delete this server?"):
             return
-
         self.try_or_error(
-            "Could not delete server.", self.dc.delete_server, server_id)
+            "Could not delete server.", self.dc.delete_server, server_id,
+        )
         print("Server deleted.")
 
     @needs_auth
